const MessagesContainerSelector = '#pro-messages';

const LoadingRowSelector = '#pro-loading-row';
const MessageRowSelector = '.pro-message-row';
const FooterRowSelector = '#pro-footer-row';

const MessageRowTemplateSelector = '#pro-message-row-template';
const MessageErrorAlertTemplate = '#pro-message-error-alert-template';
const TextFieldTemplateSelector = '#pro-text-field-template';
const TimerFieldTemplateSelector = '#pro-timer-field-template';

const MessageRowContentSelector = '.pro-message-row-content';

const MessageFormSelector = '.pro-message-form';
const NameHeaderSelector = '.pro-name-header';
const SendButtonSelector = '.pro-send-button';
const SendingMessageSelector = '.pro-sending-message';
const SuccessImageSelector = '.pro-success-image';

const FieldSelector = '.pro-field';

const FieldInputSelector = '.pro-field-input';
const FieldLabelSelector = '.pro-field-label';

const RefreshDelay = 5000;
const AnimationDuration = 200;
const HoldResultDelay = 2000;

var isInitialLoad = true;
var lastMessages = [];

$(function() {
  LoadMessages();
});

function LoadMessages() {
  Get('/html/data/messages')
    .done(function(result) {
      DisplayMessages(result);
    })
    .always(function() {
      setTimeout(LoadMessages, RefreshDelay);
    });
}

function DisplayMessages(messages) {
  if (isInitialLoad) {
    isInitialLoad = false;

    $(LoadingRowSelector).hide();
    $(FooterRowSelector).show();
  }
  else if (equal(messages, lastMessages))
    return;

  $(MessagesContainerSelector).find(MessageRowSelector).remove();

  lastMessages = messages;

  for (var i = 0; i < messages.length; ++i) {
    var message = messages[i];

    $(FooterRowSelector).before(CreateMessageRow(message.id, message.name, message.fields));
  }
}

function CreateMessageRow(id, name, fields) {
  var messageRow = $($(MessageRowTemplateSelector).html());

  messageRow.find(MessageFormSelector).prop('id', id);
  messageRow.find(NameHeaderSelector).text(name);

  BindClick(messageRow.find(SendButtonSelector), SendMessage);

  var messageForm = messageRow.find(MessageFormSelector);

  for (var i = 0; i < fields.length; ++i) {
    var field = fields[i];

    switch (field.type) {
      case 'text':
        messageForm.append(CreateTextField(field.id, field.name));
        break;

      case 'timer':
        messageForm.append(CreateTimerField(field.id, field.name, field.value));
        break;
    }
  }

  return messageRow;
};

function CreateTextField(id, name) {
  var textField = $($(TextFieldTemplateSelector).html());
  var input = textField.find(FieldInputSelector);
  var label = textField.find(FieldLabelSelector);

  SetID(input, id);

  label.prop('for', id);
  label.text(name);

  return textField;
}

function CreateTimerField(id, name, value) {
  var textField = $($(TimerFieldTemplateSelector).html());
  var input = textField.find(FieldInputSelector);
  var label = textField.find(FieldLabelSelector);

  SetID(input, id);
  input.val(value);

  label.prop('for', id);
  label.text(name);

  return textField;
}

function SendMessage(sender) {
  var sendButton = $(sender);
  var messageRow = sendButton.closest(MessageRowSelector);
  var messageForm = messageRow.find(MessageFormSelector);
  var sendingMessage = messageRow.find(SendingMessageSelector);
  var successImage = messageRow.find(SuccessImageSelector);

  var postData = {
    id: GetID(messageForm),
    fields: GetFields(messageForm)
  };

  var wasSuccess = undefined;
  var isButtonGone = false;
  var isResultDisplayed = false;

  Disable(sendButton);

  sendButton.fadeOut(AnimationDuration, function() {
    isButtonGone = true;

    sendingMessage.show();
    DisplayResult();
   });

  function DisplayResult() {
    if (isResultDisplayed)
      return;

    if (wasSuccess !== undefined && isButtonGone) {
      isResultDisplayed = true;

      sendingMessage.hide();
      Enable(sendButton);

      if (wasSuccess) {
        successImage.fadeIn(AnimationDuration, function() {
          setTimeout(function() {
            successImage.fadeOut(AnimationDuration, function() {
              sendButton.fadeIn(AnimationDuration);
            });
          }, HoldResultDelay);
        });
      }
      else {
        messageRow.find(MessageRowContentSelector).prepend($($(MessageErrorAlertTemplate).html()));
        sendButton.fadeIn(AnimationDuration);
      }
    }
  }

  Post('/html/data/messages', postData, false)
    .done(function(result) {
      wasSuccess = true;
    })
    .fail(function(err) {
      wasSuccess = false;
    })
    .always(function() {
      DisplayResult();
    });
}

function GetFields(messageForm) {
  var fields = [];

  messageForm.children(FieldSelector).each(function() {
    var input = $(this).find(FieldInputSelector);

    fields.push({
      id: GetID(input),
      value: input.val()
    });
  });

  return fields;
}
